/*
 * For the indicator to work, place the file
 * SmoothAlgorithms.mqh
 * in the directory: MetaTrader\\MQL5\Include
 */
//+X================================================================X+
//|                                                         EMAx.mq5 |
//|                               Copyright  2010, Nikolay Kositsin |
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+X================================================================X+
#property copyright "2010,   Nikolay Kositsin"
#property link      "farria@mail.redcom.ru"
#property version   "1.00"

//---- Drawing the indicator in the main window
#property indicator_chart_window
//---- one buffer is used for calculation and drawing of the indicator
#property indicator_buffers 2
//---- only one plot is used
#property indicator_plots   2
//---- drawing of the indicator as a line
#property indicator_type1   DRAW_LINE
//---- green color is used as the color of the indicator line
#property indicator_color1  LimeGreen
//---- drawing of the indicator as a line
#property indicator_type2   DRAW_LINE
//---- green color is used as the color of the indicator line
#property indicator_color2  Red

//---- input parameters of the indicator
input double EMAPeriod1 = 13; // period of EMA
input double EMAPeriod2 = 26; // period of EMA
input int    EMAShift = 0; // horizontal shift of EMA in bars
input int    EMAPriceShift = 0; // vertical shift of EMA in points

//--- indicator buffer
double ExtLineBuffer1[];
double ExtLineBuffer2[];

double dPriceShift;
//+X================================================================X+
//| Custom indicator initialization function                         |
//+X================================================================X+
void OnInit()
  {
//----+
   //---- name for the data window and for the label of sub-windows 
   string short_name = "EMA";
   IndicatorSetString(INDICATOR_SHORTNAME, short_name + "(" + DoubleToString(EMAPeriod1,2) +", " +DoubleToString(EMAPeriod2,2) + ")");
     
   //---- setting values of the indicator that won't be visible on the chart
   PlotIndexSetDouble(0, PLOT_EMPTY_VALUE, EMPTY_VALUE);
   //---- transformation of the dynamic array ExtLineBuffer into an indicator buffer
   SetIndexBuffer(0, ExtLineBuffer1, INDICATOR_DATA);
   //---- shifting the moving average horizontally by MAShift
   PlotIndexSetInteger(0, PLOT_SHIFT, EMAShift);
   
   PlotIndexSetDouble(1, PLOT_EMPTY_VALUE, EMPTY_VALUE);
   //---- transformation of the dynamic array ExtLineBuffer into an indicator buffer
   SetIndexBuffer(1, ExtLineBuffer2, INDICATOR_DATA);
   //---- shifting the moving average horizontally by MAShift
   PlotIndexSetInteger(1, PLOT_SHIFT, EMAShift);
   
   //---- Setting the format of accuracy of displaying the indicator
   IndicatorSetInteger(INDICATOR_DIGITS, _Digits + 1);
   //---- initialization of the vertical shift
   dPriceShift = _Point * EMAPriceShift;
//----+
  }
//+X================================================================X+
// Description of the class CMoving_Average                          | 
//+X================================================================X+ 
#include <SmoothAlgorithms.mqh> 
//+X================================================================X+ 
//|  Moving Average                                                  |
//+X================================================================X+
int OnCalculate
               (
                const int rates_total, // amount of history in bars at the current tick
                const int prev_calculated, // amount of history in bars at the previous tick
                const int begin, // number of beginning of bars for reliable calculation
                const double &price[] // price array for calculation of the indicator
               ) 
  {
//----+   
   //---- checking the number of bars to be enough for the calculation
   if (rates_total < begin)
    return(0);
   
   //---- declaration of local variables 
   int first, bar;
   
   //---- calculation of the starting number 'first' for the cycle of recalculation of bars
   if (prev_calculated == 0) // checking for the first start of the indicator calculation
    {
     first = begin; // starting number for calculation of all bars
    }
   else first = prev_calculated - 1; // starting number for calculation of new bars
   
   //---- declaration of variable of the class CMoving_Average from the file MASeries_Cls.mqh
   static CMoving_Average EMA[2];
   
   //---- main cycle of calculation of the indicator
   for(bar = first; bar < rates_total; bar++)
    {
     //---- Getting the average value. One call pf the EMASeries function.  
     ExtLineBuffer1[bar] = EMA[0].EMASeries(begin, prev_calculated, rates_total, EMAPeriod1, price[bar], bar, false) + dPriceShift;
     ExtLineBuffer2[bar] = EMA[1].EMASeries(begin, prev_calculated, rates_total, EMAPeriod2, price[bar], bar, false) + dPriceShift;
    }
//----+     
   return(rates_total);
  }
//+X----------------------+ <<< The End >>> +-----------------------X+
